--[[--toggle all slots selection in sequencer
 renoise.tool():add_keybinding {
  name = "Global:Tools:Toggle All Slots Selection in Sequencer",
  invoke = function() select_all_slots_in_sequencer() end
}--]]


function select_all_slots_in_sequencer()

  local song = renoise.song()
  --if 1st or last seq not selected then select all
  if song.sequencer.selection_range[1] ~= 1 or  
     song.sequencer.selection_range[2] ~= #song.sequencer.pattern_sequence then
    --set to whole range
    song.sequencer.selection_range = {1,#song.sequencer.pattern_sequence}
  else
    --set to current pattern
    song.sequencer.selection_range = {song.selected_pattern_index,song.selected_pattern_index}
  end
end

---------------------
--main whole song
---------------------
function main_whole_song()
  
  --song values  
  local song = renoise.song()
 
  --default range of current pattern
  local start_pattern = song.selected_pattern_index
  local end_pattern = song.selected_pattern_index
  
  --check if sequencer has selection then set range by it  
  if song.sequencer.selection_range[2] > 0 then
    start_pattern = song.sequencer.selection_range[1]
    end_pattern = song.sequencer.selection_range[2]
  end
    
  local seq_tracks = {}
  --get valid tracks i.e. miss groups/sends and master
  for i = 1, song.sequencer_track_count do
    if song.tracks[i].type == renoise.Track.TRACK_TYPE_SEQUENCER then
      table.insert(seq_tracks,i)
    end
  end
 
  --the pattern order gives the sometimes non-sequential order of pattern numbers in the sequencer  
  local pattern_order = {}
  for i = 1,#song.sequencer.pattern_sequence do
    table.insert(pattern_order, (song.sequencer.pattern_sequence[i]))
  end 
  
  local pattern_iter = song.pattern_iterator

  --table and incrementors
  local pattern_notes = {}
  local selected_line = 1
  local current_line = 1
 
  --loop
  for track = 1,#seq_tracks do
    
    for i = start_pattern,end_pattern do 
     
      if (i % 3) == 0 then --only relevant for multiple patterns
        coroutine.yield()
      end 
      --reset counters for each pattern
      selected_line = 1
      current_line = 1
      --status
      renoise.app():show_status("Re-Order Notes By Pitch Tool: (Re-Ordering Track: "..track.." / "..#seq_tracks..",  Sequencer Position: "..(i-1).." / "..(end_pattern-1).." Pattern no. "..(pattern_order[i]-1)..")") 
       
      --iterate track in pattern
      for pos,line in pattern_iter:note_columns_in_pattern_track(pattern_order[i],seq_tracks[track]) do
         --get each line in row values
         pattern_notes[current_line] = { line.note_value, line.instrument_value, line.volume_value, line.panning_value, line.delay_value }  
         current_line = current_line + 1
        
        --end of row
        if current_line == 13 then 
          --sort
           if not options.preserve_column_pos.value == true then
            modded_bubbleSort_left_align(pattern_notes)
          else
            modded_bubbleSort(pattern_notes)
          end
          --add in values
          local line = song.patterns[pattern_order[i]].tracks[seq_tracks[track]].lines[selected_line] 
        
          for i = 1,12 do
            line.note_columns[i]:clear()--clear current line
            line.note_columns[i].note_value = pattern_notes[i][1]--add sorted notes:
            line.note_columns[i].instrument_value = pattern_notes[i][2]
            line.note_columns[i].volume_value = pattern_notes[i][3]
            line.note_columns[i].panning_value = pattern_notes[i][4]
            line.note_columns[i].delay_value = pattern_notes[i][5] 
          end 
          --increment
          selected_line = selected_line + 1
          current_line = 1    
        end   
      end 
    end
  end
renoise.app():show_status("Re-Order Notes By Pitch Tool: (Re-Order Complete)")

end



